import ibmsecurity.utilities.tools
import logging

module_uri = "/isam/felb/configuration/ssl"
required_module = None
required_version = None

logger = logging.getLogger(__name__)


def enable(isamAppliance, keyfile, check_mode=False, force=False):
    """
    Creates ssl configuration
    """
    if force is True or _check_enable(isamAppliance, keyfile) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Creating SSL configuration", module_uri,
                                             {
                                                 "keyfile": keyfile
                                             }, requires_version=required_version, requires_modules=required_module)

    return isamAppliance.create_return_object()


def disable(isamAppliance, check_mode=False, force=False):
    """
    Deletes ssl configuration
    """
    if force is True or _check_disable(isamAppliance) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Disabling SSL", module_uri,
                                               requires_version=required_version, requires_modules=required_module)
    else:
        return isamAppliance.create_return_object()


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieves ssl configuration
    """
    return isamAppliance.invoke_get("Retrieving SSL configuration", module_uri,
                                    requires_version=required_version, requires_modules=required_module)


def update(isamAppliance, keyfile, check_mode=False, force=False):
    """
    Updating SSL configuration
    """
    # Call to check function to see if configuration already exist
    update_required = _check_enable(isamAppliance, keyfile)

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating SSL configuration", module_uri,
                                            {
                                                'keyfile': keyfile

                                            }, requires_modules=required_module, requires_version=required_version)

    return isamAppliance.create_return_object()


def _check_enable(isamAppliance, keyfile=None):
    """
    checks add function for idempotency
    """
    change_required = False
    check_obj = get(isamAppliance)
    # checks to see if ssl configuration exists
    try:
        if check_obj['data']['keyfile'] != keyfile:
            change_required = True
    except:
        change_required = True
    return change_required


def _check_disable(isamAppliance):
    """
    Checks delete function for idempotency
    """

    check_obj = get(isamAppliance)

    if check_obj['data']['enabled'] == True:
        return True
    else:
        return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

# URI for this module
uri = "/isam/application_logs"


def get_all(isamAppliance, file_path='', recursive='yes', flat_details=None, check_mode=False, force=False):
    """
    Retrieving the contents of a directory from the application log files area

    flat_details - takes 'yes' or 'no'
    file_path - use the "path" value obtained when run with flat_details = 'yes'
    """
    return isamAppliance.invoke_get("Retrieving the contents of a directory from the application log files area",
                                    "{0}/{1}{2}".format(uri, file_path,
                                                        tools.create_query_string(
                                                            recursive=recursive,
                                                            flat_details=flat_details)))


def get(isamAppliance, file_path, length=None, start=None, check_mode=False, force=False):
    """
    Retrieving the contents of a file from the application log files area
    """
    return isamAppliance.invoke_get("Retrieving the contents of a file from the application log files area",
                                    "{0}/{1}{2}".format(uri, file_path,
                                                        tools.create_query_string(
                                                            length=length,
                                                            start=start)))


def clear(isamAppliance, file_id, check_mode=False, force=False):
    """
    Clearing a log file
    """
    delete_required = False
    if force is False:
        try:
            ret_obj = get(isamAppliance, file_id, length=1, start=1)
            delete_required = True  # Exception thrown if the file is empty
        except:
            pass

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Clearing a log file",
                "{0}/{1}?action=clear".format(uri, file_id))

    return isamAppliance.create_return_object()


def delete(isamAppliance, file_id, check_mode=False, force=False):
    """
    Deleting a log file
    """
    if force is False:
        try:
            ret_obj = get(isamAppliance, file_id, length=1, start=1)
            delete_required = True  # Exception thrown if the file is empty
        except:
            delete_required = False

    if force is True or delete_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Deleting a log file",
                "{0}/{1}?action=delete".format(uri, file_id))

    return isamAppliance.create_return_object()


def export_file(isamAppliance, file_path, filename, check_mode=False, force=False):
    """
    Downloading a file from the file application log files area
    """
    import os.path

    if force is True or (os.path.exists(filename) is False):
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Downloading a file from the file application log files area",
                "{0}/{1}?type=File".format(uri, file_path),
                filename, no_headers=True)

    return isamAppliance.create_return_object()



import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/logging"
requires_modules = None
requires_versions = None


def get(isamAppliance):
    """
    Retrieves logging configuration attributes
    """
    return isamAppliance.invoke_get("Retrieving logging configuration", module_uri)


def update(isamAppliance, local, remote_address, remote_port, remote_facility, check_mode=False, force=False):
    """
    Updates logging configuration
    """
    change_required, json_data = _check(isamAppliance, local, remote_address, remote_port, remote_facility)

    if force is True or change_required is True:
        return isamAppliance.invoke_put("Updating Configuration", module_uri, json_data,
                                        requires_modules=requires_modules, requires_version=requires_versions)
    if change_required is False:
        return isamAppliance.create_return_object(changed=False)


def _check(isamAppliance, local, remote_address, remote_port, remote_facility):
    """
    Check for idempotency
    """
    check_obj = get(isamAppliance)
    change_required = False
    # If the configuration is local, remote entries are not used
    if local is True:
        json_data = {
            "local": True,
            "remote_address": "",
            "remote_port": None,
            "remote_facility": None

        }
        if check_obj['data']['local'] != local:
            change_required = True
        return change_required, json_data
    else:
        if check_obj['data']['remote_address'] != remote_address:
            change_required = True
        if check_obj['data']['remote_port'] != remote_port:
            change_required = True
        if check_obj['data']['remote_facility'] != remote_facility:
            change_required = True

        json_data = {
            "local": local,
            "remote_address": remote_address,
            "remote_port": remote_port,
            "remote_facility": remote_facility
        }

        return change_required, json_data



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve existing suffixes.
    """
    return isamAppliance.invoke_get("Retrieving suffixes from embedded ldap",
                                    "/isam/embedded_ldap/suffixes/v1")


def add(isamAppliance, name, check_mode=False, force=False):
    """
    Add a suffix to embedded ldap
    """
    exists = False
    id = None

    if force is False:
        exists, id = _check(isamAppliance, name)

    if force is True or exists is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post(
                "Add suffix to embedded ldap",
                "/isam/embedded_ldap/suffixes/v1",
                {'name': name})

    return isamAppliance.create_return_object()


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    Add a suffix to embedded ldap
    """
    exists = False
    id = None

    if force is False:
        exists, id = _check(isamAppliance, name)

    if force is True or exists is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Delete suffix from embedded ldap",
                "/isam/embedded_ldap/suffixes/{0}/v1".format(id))

    return isamAppliance.create_return_object()


def _check(isamAppliance, name):
    """
    Check if suffix exists
    """
    ret_obj = get(isamAppliance)

    for suffix in ret_obj['data']:
        if suffix['name'] == name:
            logger.info("Suffix found in embedded ldap: " + name)
            return True, suffix['id']

    logger.info("Suffix *not* found in embedded ldap: " + name)
    return False, None


def compare(isamAppliance1, isamAppliance2):
    """
    Compare suffixes in two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging

from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/attributes"
requires_modules = None
requires_version = None


def add(isamAppliance, name, value, check_mode=False, force=False):
    """
    Creates attribute
    """
    if force is True or _check(isamAppliance, name, value) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Creating Attribute", module_uri,
                                             {
                                                 "name": name,
                                                 "value": value
                                             }, requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def delete(isamAppliance, attribute_name, check_mode=False, force=False):
    """
    deletes given attribute
    """
    if force or search(isamAppliance, attribute_name):
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting Attribute", "{0}/{1}".format(module_uri, attribute_name),
                                               requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def get(isamAppliance, attribute_name):
    """
    Retrieves attribute
    """
    return isamAppliance.invoke_get("Retrieving Attribute", "{0}/{1}".format(module_uri, attribute_name),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_all(isamAppliance):
    """
    Retrieves all attributes
    """
    return isamAppliance.invoke_get("Retrieving Attributes", "{0}/".format(module_uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def update(isamAppliance, attribute_name, value, check_mode=False, force=False):
    """
    Updates given attribute
    """

    if force is True or _check(isamAppliance, attribute_name, value) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating Attribute", "{0}/{1}".format(module_uri, attribute_name),
                                            {
                                                "value": value
                                            }, requires_version=requires_version, requires_modules=requires_modules)

    return isamAppliance.create_return_object(changed=False)


def _check(isamAppliance, attribute_name, attribute_value):
    """
    Check for idempotency
    """
    # Error handling to see if attribute exist returns True if attribute doesnt exist
    try:
        temp_obj = get(isamAppliance, attribute_name)
    except:
        return True

    if temp_obj['data']['value'] != attribute_value:
        return True
    else:
        return False


def search(isamAppliance, attribute_name):
    """
    Check for idempotency
    """
    # Error handling to see if attribute exist returns True if attribute doesnt exist
    try:
        return get(isamAppliance, attribute_name)
    except:
        return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare access policies between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    obj1 = {'rc': 0, 'data': []}
    obj2 = {'rc': 0, 'data': []}

    for attr in ret_obj1["data"]:
        if search(isamAppliance=isamAppliance1, attribute_name=attr["name"]):
            value = get(isamAppliance1, attribute_name=attr["name"])
            obj1['data'].append({attr["name"]: value["data"]["value"]})

    for attr in ret_obj2["data"]:
        if search(isamAppliance=isamAppliance2, attribute_name=attr["name"]):
            value = get(isamAppliance2, attribute_name=attr["name"])
            obj2['data'].append({attr["name"]: value["data"]["value"]})

    return tools.json_compare(obj1, obj2)



import logging
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)
uri = "/mga/server_connections/isamruntime"
requires_modules = ["mga", "federation"]
requires_version = "9.0.5.0"


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieving a list of all ISAM Runtime server connections
    """
    return isamAppliance.invoke_get("Retrieving a list of all ISAM Runtime server connections",
                                    "{0}/v1".format(uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get(isamAppliance, name, check_mode=False, force=False):
    """
    Retrieving an ISAM Runtime server connection
    """

    ret_obj = search(isamAppliance, name=name)
    id = ret_obj['data']

    if id == {}:
        return isamAppliance.create_return_object()
    else:
        return isamAppliance.invoke_get("Retrieving an ISAM Runtime server connection",
                                        "{0}/{1}/v1".format(uri, id),
                                        requires_modules=requires_modules, requires_version=requires_version)


def add(isamAppliance, name, connection, description="", locked=False, check_mode=False, force=False):
    """
    Creating an ISAM Runtime server connection
    """

    if force is True or _check_exists(isamAppliance, name=name) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post(
                "Creating an ISAM Runtime server connection",
                "{0}/v1".format(uri),
                {
                    "locked": locked,
                    "description": description,
                    "type": "isamruntime",
                    "connection": connection,
                    "name": name
                },
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()


def update(isamAppliance, name, connection, locked=False, description='', new_name=None, check_mode=False, force=False):
    """
    Modifying an ISAM Runtime server connection

    Use new_name to rename the connection, cannot compare password so update will take place everytime

    """

    if force is True or _check_exists(isamAppliance, name):
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            json_data = _create_json(name=name, description=description, locked=locked, connection=connection)
            if new_name is not None:  # Rename condition
                json_data['name'] = new_name

            ret_obj = search(isamAppliance, name=name)
            id = ret_obj['data']

            return isamAppliance.invoke_put(
                "Modifying an ISAM Runtime server connection",
                "{0}/{1}/v1".format(uri, id),
                json_data,
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()


def set(isamAppliance, name, locked=False, connection=None, description=None, new_name=None,
        check_mode=False, force=False):
    """
    Creating or Modifying a isamruntime
    """
    ret_obj = search(isamAppliance, name=name)
    id = ret_obj['data']

    if id == {}:
        # If no uuid was found, Force the add
        return add(isamAppliance, name=name, locked=locked, connection=connection, description=description,
                   check_mode=check_mode, force=True)
    else:
        # Update isamruntime
        return update(isamAppliance, name=name, locked=locked, connection=connection, description=description,
                      new_name=new_name, check_mode=check_mode, force=force)


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    Deleting an ISAM Runtime server connection

    """
    ret_obj = search(isamAppliance, name, check_mode=False, force=False)
    id = ret_obj['data']

    if force is True or id != {}:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_delete(
                "Deleting an ISAM Runtime server connection",
                "{0}/{1}/v1".format(uri, id),
                requires_modules=requires_modules, requires_version=requires_version
            )

    return isamAppliance.create_return_object()


def compare(isamAppliance1, isamAppliance2):
    """
    Compare isamruntime between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['uuid']
    for obj in ret_obj2['data']:
        del obj['uuid']

    return tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['uuid'])


def search(isamAppliance, name, force=False, check_mode=False):
    """
    Retrieve ID for isamruntime
    """
    ret_obj = get_all(isamAppliance)

    ret_obj_new = isamAppliance.create_return_object()

    for obj in ret_obj['data']:
        if obj['name'] == name:
            logger.info("Found isamruntime '{0}' uuid: '{1}'".format(name, obj['uuid']))
            ret_obj_new['data'] = obj['uuid']

    return ret_obj_new


def _check_exists(isamAppliance, name=None, id=None):
    """
    Check if ISAM runtime Connection already exists
    """
    ret_obj = get_all(isamAppliance)

    for obj in ret_obj['data']:
        if (name is not None and obj['name'] == name) or (id is not None and obj['uuid'] == id):
            return True

    return False


def _create_json(name, connection, description, locked):
    """
    Create a JSON to be used for the REST API call
    """
    json = {
        "connection": connection,
        "type": "isamruntime",
        "name": name,
        "description": description,
        "locked": locked
    }

    return json



import logging

logger = logging.getLogger(__name__)

requires_model = "Appliance"


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieve the cluster manager log file names
    """
    return isamAppliance.invoke_get("Retrieve the cluster manager log file names",
                                    "/isam/cluster/logging/v1", requires_model=requires_model)


def get(isamAppliance, file_id, size=100, start=None, options=None, check_mode=False, force=False):
    """
    Retrieve a log file snippet
    """
    return isamAppliance.invoke_get("Retrieve a log file snippet",
                                    "/isam/cluster/logging/{0}/v1".format(file_id), requires_model=requires_model)


def _check(isamAppliance, file_id):
    check_obj = {'vale': False, 'warnings': ""}

    ret_obj = get(isamAppliance, file_id)
    check_obj['warnings'] = ret_obj['warnings']

    if ret_obj['data'] != {}:
        if ret_obj['data']['contents'] == '':
            check_obj['value'] = False
            return check_obj
        else:
            check_obj['value'] = True
            return check_obj
    else:
        check_obj['value'] = False
        return check_obj


def delete(isamAppliance, file_id, check_mode=False, force=False):
    """
    Clear a log file
    """
    check_obj = _check(isamAppliance, file_id)

    if force is True or check_obj['value'] is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=check_obj['warnings'] )
        else:
            return isamAppliance.invoke_delete(
                "Clear a log file",
                "/isam/cluster/logging/{0}/v1".format(file_id), requires_model=requires_model, warnings=check_obj['warnings'])

    return isamAppliance.create_return_object(warnings=check_obj['warnings'])


def export_file(isamAppliance, file_id, filename, check_mode=False, force=False):
    """
    Export a cluster manager log file
    """
    import os.path

    if force is True or os.path.exists(filename) is False:
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Export a cluster manager log file",
                "/isam/cluster/logging/{0}/v1?export".format(file_id),
                filename, requires_model=requires_model)

    return isamAppliance.create_return_object()



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/attributes"
requires_modules = None
requires_version = None


def add(isamAppliance, name, value, check_mode=False, force=False):
    """
    Creates attribute
    """
    if force is True or _check_add(isamAppliance, name, value) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Creating an attribute", module_uri,
                                             {
                                                 "name": name,
                                                 "value": value
                                             }, requires_modules=requires_modules, requires_version=requires_version)

    else:
        return isamAppliance.create_return_object()


def set(isamAppliance, name, value=None, check_mode=False, force=False):
    """
    Determines if add or update is called
    """
    try:
        ret_obj = get(isamAppliance, name)
    except:
        return add(isamAppliance, name, value, check_mode, force)

    return update(isamAppliance, name, value, check_mode, force)


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    deletes given attribute
    """
    if force is True or _check_delete(isamAppliance, name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting an attribute", "{0}/{1}".format(module_uri, name),
                                               requires_modules=requires_modules, requires_version=requires_version)
    else:
        return isamAppliance.create_return_object()


def get(isamAppliance, attribute_name):
    """
    Retrieves attribute
    """
    return isamAppliance.invoke_get("Retrieving an attribute", "{0}/{1}".format(module_uri, attribute_name),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_all(isamAppliance):
    """
    Retrieves all attributes
    """

    return isamAppliance.invoke_get("Retrieving attribute names", "{0}/".format(module_uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def update(isamAppliance, attribute_name, value, check_mode=False, force=False):
    """
    Updates given attribute
    """

    if force is True or _check(isamAppliance, attribute_name, value) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating an attribute", "{0}/{1}".format(module_uri, attribute_name),
                                            {
                                                "value": value
                                            }, requires_version=requires_version, requires_modules=requires_modules)

    else:
        return isamAppliance.create_return_object()


def _check(isamAppliance, attribute_name, attribute_value):
    """
    Check for idempotency
    """

    # Error handling to see if attribute exist returns True if attribute doesnt exist
    try:
        temp_obj = get(isamAppliance, attribute_name)
    except:
        return True

    if temp_obj['data']['value'] != attribute_value:
        return True
    else:
        return False


def _check_delete(isamAppliance, attribute_name):
    """
    idempotency check for delete
    """

    # check to see if attribute name exist
    try:
        temp = get(isamAppliance, attribute_name)
    except:
        return False

    return True


def _check_add(isamAppliance, name, value):
    """
    checks add function for idempotency
    """
    check_obj = {}

    try:
        check_obj = get(isamAppliance, name)
    except:

        return True

    return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/errorpages/"
requires_modules = None
requires_version = None


def get(isamAppliance, error_page, check_mode=False, force=False):
    """
    Retrieving an error page
    """
    return isamAppliance.invoke_get("Retrieving an error page", "{0}{1}".format(module_uri, error_page),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieving List of Error Pages
    """
    return isamAppliance.invoke_get("Retrieving List of Error Pages", module_uri,
                                    requires_modules=requires_modules, requires_version=requires_version)


def set(isamAppliance, error_page, contents, check_mode=False, force=False):
    """
    Updating an error page
    """
    if force is False:
        ret_obj = get(isamAppliance, error_page)

    if force is True or ret_obj['data'] != contents:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating an error page", "{0}{1}".format(module_uri, error_page),
                                            {
                                                "contents": contents
                                            }, requires_version=requires_version, requires_module=requires_modules)
    else:
        return isamAppliance.create_return_object()


def import_file(isamAppliance, error_page, file, check_mode=False, force=False):
    """
    Uploading an error page
    """
    if force is True or _check_upload(isamAppliance, error_page, file) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post_files(description="Uploading an error page",
                                                   uri="{0}{1}".format(module_uri, error_page),
                                                   fileinfo=[{
                                                       'file_formfield': 'file',
                                                       'filename': file,
                                                       'mimetype': 'application/octet-stream'
                                                   }],
                                                   data={},
                                                   requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def export_file(isamAppliance, error_page, filename, check_mode=False, force=False):
    """
    Downloading an error page
    """
    import os.path
    if force is True or os.path.exists(error_page) is False:
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Downloading an error page",
                "{0}{1}?export=true".format(module_uri, error_page),
                filename=filename
            )

    return isamAppliance.create_return_object()


def set(isamAppliance, error_page, content=None, check_mode=False, force=False):
    """
    determines if add or update is used
    """
    try:
        check_obj = get(isamAppliance, error_page)
    except:
        # if error_page does not exist, function calls add function
        logger.debug("Error Page configuration does not exist")
        return add(isamAppliance, error_page, check_mode, force)

    return update(isamAppliance, error_page, content, check_mode, force)


def compare(isamAppliance1, isamAppliance2):
    """
    Compare error pages between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/services/"
requires_modulers = None
requires_version = None


def get(isamAppliance, service_name, check_mode=False, force=False):
    """
    Retrieves layer configuration
    """
    return isamAppliance.invoke_get("Retrieving Layer Configuration", "{0}{1}/layer".format(module_uri, service_name),
                                    requires_version=requires_version, requires_modules=requires_modulers)


def update(isamAppliance, service_name, type, layer7_secure, layer7_ssl_label, layer7_cookie=None, check_mode=False,
           force=False):
    """
    Updates specified service name layer
    """
    change_required = _check(isamAppliance, service_name, type, layer7_secure, layer7_ssl_label, layer7_cookie)

    if force is True or change_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating Service Layer", "{0}{1}/layer".format(module_uri, service_name),
                                            {
                                                "type": type,
                                                "layer7_secure": layer7_secure,
                                                "layer7_ssl_label": layer7_ssl_label,
                                                "layer7_cookie": layer7_cookie

                                            }, requires_version=requires_version, requires_modules=requires_modulers)
    else:
        return isamAppliance.create_return_object()


def _check(isamAppliance, service_name, type, layer7_secure, layer7_ssl_label, layer7_cookie):
    """
    Checks update for idempotency
    """

    ret_obj = get(isamAppliance, service_name)
    if ret_obj['data']['type'] != type:
        return True
    elif ret_obj['data']['layer7_secure'] != layer7_secure:
        return True
    elif ret_obj['data']['layer7_ssl_label'] != layer7_ssl_label:
        return True
    elif ret_obj['data']['layer7_cookie'] != layer7_cookie:
        return True
    else:
        return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/ha"
requires_modules = None
requires_version = None


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving HA configuration
    """
    return isamAppliance.invoke_get("Retrieving HA configuration", module_uri,
                                    requires_version=requires_version, requires_modules=requires_modules)


def disable(isamAppliance, check_mode=False, force=False):
    """
    Disabling HA
    """
    if force is True or _check_disable(isamAppliance) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Disabling HA", module_uri,
                                               requires_version=requires_version,
                                               requires_modules=requires_modules)
    else:
        return isamAppliance.create_return_object()


def enable(isamAppliance, is_primary, interface, remote, port, health_check_interval,
           health_check_timeout, check_mode=False, force=False):
    """
    Enabling HA
    """
    if force is True or _check_enable(isamAppliance, is_primary, interface, remote, port, health_check_interval,
                                      health_check_timeout) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Enabling HA", module_uri,
                                             {
                                                 "is_primary": is_primary,
                                                 "interface": interface,
                                                 "remote": remote,
                                                 "port": port,
                                                 "health_check_interval": health_check_interval,
                                                 "health_check_timeout": health_check_timeout
                                             }, requires_version=requires_version, requires_modules=requires_modules)
    else:
        return isamAppliance.create_return_object()


def update(isamAppliance, is_primary, interface, remote, port, health_check_interval,
           health_check_timeout, check_mode=False, force=False):
    """
    Updating HA configuration
    """
    # Call to check function to see if configuration already exist
    update_required = _check_enable(isamAppliance, is_primary=is_primary, interface=interface, remote=remote,
                                    port=port, health_check_interval=health_check_interval,
                                    health_check_timeout=health_check_timeout)

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating HA configuration", module_uri,
                                            {
                                                "is_primary": is_primary,
                                                "interface": interface,
                                                "remote": remote,
                                                "port": port,
                                                "health_check_interval": health_check_interval,
                                                "health_check_timeout": health_check_timeout

                                            }, requires_modules=requires_modules, requires_version=requires_version)
    else:
        return isamAppliance.create_return_object()


def set(isamAppliance, is_primary, interface, remote, port, health_check_interval,
        health_check_timeout, check_mode=False, force=False):
    """
    determines if add or update is used.
    """
    check_obj = get(isamAppliance)

    if check_mode['data']['enabled'] is False:
        enable(isamAppliance, is_primary, interface, remote, port, health_check_interval,
               health_check_timeout, check_mode, force)

    else:
        update(isamAppliance, is_primary, interface, remote, port, health_check_interval,
               health_check_timeout, check_mode, force)


def _check_enable(isamAppliance, is_primary, interface, remote, port, health_check_interval,
                  health_check_timeout):
    """
    idempotency test for each parameter
    """
    ret_obj = get(isamAppliance)
    if ret_obj['data']['enabled'] != True:
        return True
    elif ret_obj['data']['is_primary'] != is_primary:
        return True
    elif ret_obj['data']['interface'] != interface:
        return True
    elif ret_obj['data']['remote'] != remote:
        return True
    elif ret_obj['data']['port'] != port:
        return True
    elif ret_obj['data']['health_check_interval'] != health_check_interval:
        return True
    elif ret_obj['data']['health_check_timeout'] != health_check_timeout:
        return True
    else:
        return False


def _check_disable(isamAppliance):
    """
    Checks delete function for idempotency
    """
    check_obj = get(isamAppliance)

    if check_obj['data']['enabled'] == True:
        return True
    else:
        return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare FELB HA configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import copy
import ibmsecurity.utilities.tools
from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)

uri = "/isam/cluster/v2"
requires_modules = None
requires_version = None
requires_model = "Appliance"

try:
    basestring
except NameError:
    basestring = (str, bytes)


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve the current cluster configuration
    """
    return isamAppliance.invoke_get("Retrieve the current cluster configuration", uri,
                                    requires_modules=requires_modules, requires_version=requires_version,
                                    requires_model=requires_model)


def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master_ere=None, tertiary_master=None,
        quaternary_master=None, dsc_external_clients=False, dsc_port=None, dsc_use_ssl=None, dsc_ssl_keyfile=None,
        dsc_ssl_label=None, dsc_worker_threads=64, dsc_maximum_session_lifetime=3600, dsc_client_grace_period=600,
        hvdb_embedded=True, hvdb_max_size=None, hvdb_db_type=None, hvdb_address=None, hvdb_port=None, hvdb_user=None,
        hvdb_password=None, hvdb_db2_alt_address=None, hvdb_db2_alt_port=None, hvdb_db_name=None, hvdb_db_secure=None,
        hvdb_driver_type=None, hvdb_solid_tc=None, cfgdb_embedded=True, cfgdb_db_type=None, cfgdb_address=None,
        cfgdb_port=None, cfgdb_user=None, cfgdb_password=None, cfgdb_db2_alt_address=None, cfgdb_db2_alt_port=None,
        cfgdb_db_name=None, cfgdb_db_secure=None, cfgdb_driver_type=None, cfgdb_solid_tc=None, first_port=2020,
        cfgdb_fs=None, dsc_trace_level=None, ignore_password_for_idempotency=False, check_mode=False, force=False):
    """
    Set cluster configuration
    """

    warnings = []
    # Create a simple json with just the main client attributes
    cluster_json = {
        "primary_master": primary_master,
        "dsc_external_clients": dsc_external_clients,
        "dsc_worker_threads": dsc_worker_threads,
        "dsc_maximum_session_lifetime": dsc_maximum_session_lifetime,
        "dsc_client_grace_period": dsc_client_grace_period,
        "hvdb_embedded": hvdb_embedded,
        "cfgdb_embedded": cfgdb_embedded,
        "first_port": first_port
    }
    # Add attributes that have been supplied... otherwise skip them.
    if secondary_master is not None:
        cluster_json["secondary_master"] = secondary_master
    if master_ere is not None:
        cluster_json["master_ere"] = master_ere
    if tertiary_master is not None:
        cluster_json["tertiary_master"] = tertiary_master
    if quaternary_master is not None:
        cluster_json["quaternary_master"] = quaternary_master
    if dsc_port is not None:
        cluster_json["dsc_port"] = dsc_port
    if dsc_use_ssl is not None:
        cluster_json["dsc_use_ssl"] = dsc_use_ssl
    if dsc_ssl_keyfile is not None:
        cluster_json["dsc_ssl_keyfile"] = dsc_ssl_keyfile
    if dsc_ssl_label is not None:
        cluster_json["dsc_ssl_label"] = dsc_ssl_label
    if hvdb_max_size is not None:
        cluster_json["hvdb_max_size"] = hvdb_max_size
    if hvdb_db_type is not None:
        cluster_json["hvdb_db_type"] = hvdb_db_type
    if hvdb_address is not None:
        cluster_json["hvdb_address"] = hvdb_address
    if hvdb_port is not None:
        cluster_json["hvdb_port"] = hvdb_port
    if hvdb_user is not None:
        cluster_json["hvdb_user"] = hvdb_user
    if hvdb_password is not None:
        if ignore_password_for_idempotency:
            warnings.append("Request made to ignore hvdb_password for idempotency check.")
        else:
            warnings.append("Since existing hvdb_password cannot be read - this call will not be idempotent.")
        cluster_json["hvdb_password"] = hvdb_password
    if hvdb_db2_alt_address is not None:
        cluster_json["hvdb_db2_alt_address"] = hvdb_db2_alt_address
    if hvdb_db2_alt_port is not None:
        cluster_json["hvdb_db2_alt_port"] = hvdb_db2_alt_port
    if hvdb_db_name is not None:
        cluster_json["hvdb_db_name"] = hvdb_db_name
    if hvdb_db_secure is not None:
        cluster_json["hvdb_db_secure"] = hvdb_db_secure
    if hvdb_driver_type is not None:
        cluster_json["hvdb_driver_type"] = hvdb_driver_type
    if hvdb_solid_tc is not None:
        if (isinstance(hvdb_solid_tc, basestring)):
            import ast
            hvdb_solid_tc = ast.literal_eval(hvdb_solid_tc)
        cluster_json["hvdb_solid_tc"] = hvdb_solid_tc
    if cfgdb_db_type is not None:
        cluster_json["cfgdb_db_type"] = cfgdb_db_type
    if cfgdb_address is not None:
        cluster_json["cfgdb_address"] = cfgdb_address
    if cfgdb_port is not None:
        cluster_json["cfgdb_port"] = cfgdb_port
    if cfgdb_user is not None:
        cluster_json["cfgdb_user"] = cfgdb_user
    if cfgdb_password is not None:
        if ignore_password_for_idempotency:
            warnings.append("Request made to ignore cfgdb_password for idempotency check.")
        else:
            warnings.append("Since existing cfgdb_password cannot be read - this parameter is ignored for idempotency.")
        cluster_json["cfgdb_password"] = cfgdb_password
    if cfgdb_db2_alt_address is not None:
        cluster_json["cfgdb_db2_alt_address"] = cfgdb_db2_alt_address
    if cfgdb_db2_alt_port is not None:
        cluster_json["cfgdb_db2_alt_port"] = cfgdb_db2_alt_port
    if cfgdb_db_name is not None:
        cluster_json["cfgdb_db_name"] = cfgdb_db_name
    if cfgdb_db_secure is not None:
        cluster_json["cfgdb_db_secure"] = cfgdb_db_secure
    if cfgdb_driver_type is not None:
        cluster_json["cfgdb_driver_type"] = cfgdb_driver_type
    if cfgdb_solid_tc is not None:
        if (isinstance(cfgdb_solid_tc, basestring)):
            import ast
            cfgdb_solid_tc = ast.literal_eval(cfgdb_solid_tc)
        cluster_json["cfgdb_solid_tc"] = cfgdb_solid_tc
    if cfgdb_fs is not None:
        if ibmsecurity.utilities.tools.version_compare(isamAppliance.facts["version"], "9.0.2.0") < 0:
            warnings.append(
                "Appliance at version: {0}, cfgdb_fs: {1} is not supported. Needs 9.0.2.0 or higher. Ignoring cfgdb_fs for this call.".format(
                    isamAppliance.facts["version"], cfgdb_fs))
        else:
            cluster_json["cfgdb_fs"] = cfgdb_fs
    if dsc_trace_level is not None:
        cluster_json["dsc_trace_level"] = dsc_trace_level

    check_obj =  _check(isamAppliance, cluster_json, ignore_password_for_idempotency)
    warnings.append(check_obj['warnings'])
    if force is True or check_obj['value'] is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_post("Set cluster configuration", uri, cluster_json,
                                             requires_modules=requires_modules, requires_version=requires_version,
                                             requires_model=requires_model,
                                             warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def _check(isamAppliance, cluster_json, ignore_password_for_idempotency):
    """
    Check if provided json values match the configuration on appliance

    :param isamAppliance:
    :param cluster_json:
    :return:
    """

    check_obj = {'value': True, 'warnings':""}

    ret_obj = get(isamAppliance)
    check_obj['warnings'] = ret_obj['warnings']
    sorted_ret_obj = tools.json_sort(ret_obj['data'])
    sorted_json_data = tools.json_sort(cluster_json)
    logger.debug("Sorted Existing Data:{0}".format(sorted_ret_obj))
    logger.debug("Sorted Desired  Data:{0}".format(sorted_json_data))

    if ignore_password_for_idempotency:
        temp = copy.deepcopy(
            cluster_json)  # deep copy neccessary: otherwise password parameter would be removed from desired config dict 'cluster_json'. Comparison is done with temp<>ret_obj object
        for idx, x in enumerate(cluster_json):
            if "password" in x:
                logger.debug("Ignoring JSON password entry: '{0}' to satisfy idempotency.".format(x))
                del temp[x]
        logger.debug("Passwordless JSON to Apply: {0}".format(temp))
    else:
        temp = cluster_json

    for key, value in temp.items():
        try:
            if isinstance(value, list):
                if ibmsecurity.utilities.tools.json_sort(
                        ret_obj['data'][key] != ibmsecurity.utilities.tools.json_sort(value)):
                    logger.debug(
                        "For key: {0}, values: {1} and {2} do not match.".format(key, value, ret_obj['data'][key]))
                    check_obj['value']=False
                    return check_obj
            else:
                if ret_obj['data'][key] != value:
                    logger.debug(
                        "For key: {0}, values: {1} and {2} do not match.".format(key, value, ret_obj['data'][key]))
                    check_obj['value']=False
                    return check_obj
        except:  # In case there is an error looking up the key in existing configuration (missing)
            logger.debug("Exception processing Key: {0} Value: {1} - missing key in current config?".format(key, value))
            check_obj['value']=False
            return check_obj

    logger.debug("JSON provided already is contained in current appliance configuration.")
    check_obj['value']=True
    return check_obj


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """

    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieving a list of all SMTP server connections
    """
    return isamAppliance.invoke_get("Retrieving a list of all SMTP server connections",
                                    "/mga/server_connections/smtp/v1")


def get(isamAppliance, name, check_mode=False, force=False):
    """
    Retrieving a SMTP server connection
    """
    ret_obj = search(isamAppliance, name=name)
    id = ret_obj['data']

    if id == {}:
        return isamAppliance.create_return_object()
    else:
        return isamAppliance.invoke_get("Retrieving a SMTP server connection",
                                        "/mga/server_connections/smtp/{0}/v1".format(id))


def set(isamAppliance, name, connection, description='', locked=False, connectionManager=None, new_name=None,
        check_mode=False, force=False):
    """
    Creating or Modifying an SMTP server connection
    """
    if _check_exists(isamAppliance, name=name) is False:
        # Force the add - we already know connection does not exist
        return add(isamAppliance=isamAppliance, name=name, connection=connection, description=description,
                   locked=locked, connectionManager=connectionManager, check_mode=check_mode, force=True)
    else:
        # Update request
        return update(isamAppliance=isamAppliance, name=name, connection=connection, description=description,
                      locked=locked, connectionManager=connectionManager, new_name=new_name,
                      check_mode=check_mode, force=force)


def add(isamAppliance, name, connection, description='', locked=False, connectionManager=None, check_mode=False,
        force=False):
    """
    Creating a SMTP server connection
    """
    if force is True or _check_exists(isamAppliance, name=name) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post(
                "Creating a SMTP server connection",
                "/mga/server_connections/smtp/v1",
                _create_json(name=name, description=description, locked=locked, connection=connection,
                             connectionManager=connectionManager))

    return isamAppliance.create_return_object()


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    Deleting a SMTP server connection
    """
    if force is True or _check_exists(isamAppliance, name=name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            ret_obj = search(isamAppliance, name=name)
            id = ret_obj['data']
            return isamAppliance.invoke_delete(
                "Deleting a SMTP server connection",
                "/mga/server_connections/smtp/{0}/v1".format(id))

    return isamAppliance.create_return_object()


def update(isamAppliance, name, connection, description='', locked=False, connectionManager=None, new_name=None,
           check_mode=False, force=False):
    """
    Modifying a SMTP server connection

    Use new_name to rename the connection, cannot compare password so update will take place everytime
    """

    if force is True or _check_exists(isamAppliance, name):
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            json_data = _create_json(name=name, description=description, locked=locked, connection=connection,
                                     connectionManager=connectionManager)
            if new_name is not None:  # Rename condition
                json_data['name'] = new_name

            ret_obj = search(isamAppliance, name=name)
            id = ret_obj['data']

            return isamAppliance.invoke_put(
                "Modifying a SMTP server connection",
                "/mga/server_connections/smtp/{0}/v1".format(id), json_data)

    return isamAppliance.create_return_object()


def _create_json(name, description, locked, connection, connectionManager):
    """
    Create a JSON to be used for the REST API call
    """
    json = {
        "connection": connection,
        "type": "smtp",
        "name": name,
        "description": description,
        "locked": locked
    }
    # connection manager is optional
    if connectionManager is not None:
        json['connectionManager'] = connectionManager

    return json


def search(isamAppliance, name):
    """
    Retrieve UUID for named SMTP connection
    """
    ret_obj = get_all(isamAppliance)

    ret_obj_new = isamAppliance.create_return_object()

    for obj in ret_obj['data']:
        if obj['name'] == name:
            ret_obj_new['data'] = obj['uuid']

    return ret_obj_new


def _check_exists(isamAppliance, name=None, id=None):
    """
    Check if SMTP Connection already exists
    """
    ret_obj = get_all(isamAppliance)

    for obj in ret_obj['data']:
        if (name is not None and obj['name'] == name) or (id is not None and obj['uuid'] == id):
            return True

    return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare SMTP Connections between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['uuid']
    for obj in ret_obj2['data']:
        del obj['uuid']

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['uuid'])



import logging
import ibmsecurity.utilities.tools
from ibmsecurity.isam.aac.api_protection import definitions

logger = logging.getLogger(__name__)

# URI for this module
uri = "/wga/rsa_config"
requires_modules = ["wga"]
requires_version = None


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieve RSA Securid Configuration
    """
    return isamAppliance.invoke_get("Retrieve RSA Securid Configuration", uri, requires_modules=requires_modules,
                                    requires_version=requires_version)


def upload(isamAppliance, filename, check_mode=False, force=False):
    """
    Upload a RSA Securid Config file
    """
    warnings = [
        "Idempotency check is only to see if there was a config already uploaded. Force upload to replace existing configuration."]
    if force is True or _check(isamAppliance) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_post_files(
                "Upload a RSA Securid Config file",
                "{0}/server_config".format(uri),
                [
                    {
                        'file_formfield': 'server_config_file',
                        'filename': filename,
                        'mimetype': 'application/octet-stream'
                    }
                ],
                {}, requires_modules=requires_modules,
                requires_version=requires_version, warnings=warnings)

    return isamAppliance.create_return_object()


def _check(isamAppliance):
    ret_obj = get(isamAppliance)
    return ret_obj['data']['server_config'] == 'available'


def test(isamAppliance, username, passcode, check_mode=False, force=False):
    """
    Test RSA Configuration with username/passcode
    """
    if _check(isamAppliance) is False:
        return isamAppliance.create_return_object(warnings=["Valid configuration not found, test skipped."])
    else:
        ret_obj = isamAppliance.invoke_post("Test RSA Configuration with username/passcode", "{0}/test".format(uri),
                                            {
                                                'username': username,
                                                'passcode': passcode
                                            },
                                            requires_modules=requires_modules,
                                            requires_version=requires_version, ignore_error=True)
        if ret_obj['changed'] is True:
            ret_obj['changed'] = False

        return ret_obj


def delete(isamAppliance, check_mode=False, force=False):
    """
    Deleting or Clear RSA Securid Configuration
    """
    if force is True or _check(isamAppliance) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting or Clear RSA Securid Configuration",
                                               "{0}/server_config".format(uri),
                                               requires_modules=requires_modules,
                                               requires_version=requires_version)

    return isamAppliance.create_return_object()


def clear(isamAppliance, check_mode=False, force=False):
    """
    Clear the node secret file

    :param isamAppliance:
    :param check_mode:
    :param force:
    :return:
    """
    # TODO: This function has not been tested.  Please open an issue on GitHub if you find a problem.

    if force is True or _check(isamAppliance) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Clear the node secret file",
                                               "{0}/node_secret".format(uri),
                                               requires_modules=requires_modules,
                                               requires_version=requires_version)

    return isamAppliance.create_return_object()



import logging

from ibmsecurity.utilities import tools

logger = logging.getLogger(__name__)
uri = "/isam/dsc/admin/replicas"
requires_modules = None
requires_version = None


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the list of replica sets
    """
    return isamAppliance.invoke_get("Retrieving the list of replica sets",
                                    "{0}".format(uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_servers(isamAppliance, replica, check_mode=False, force=False):
    """
    Retrieving the list of servers for a replica set
    """
    return isamAppliance.invoke_get("Retrieving the list of servers for a replica set",
                                    "{0}/{1}/servers".format(uri, replica),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get_session(isamAppliance, replica, user="*", max="1024", check_mode=False, force=False):
    """
    Searching for session within a replica set
    """
    return isamAppliance.invoke_get("Searching for session within a replica set",
                                    "{0}/{1}/sessions{2}".format(uri, replica,
                                                                 tools.create_query_string(user=user, max=max)),
                                    requires_modules=requires_modules, requires_version=requires_version)


def delete(isamAppliance, replica, session, check_mode=False, force=False):
    """
    Terminating a session
    """

    if force is True or search(isamAppliance=isamAppliance, replica=replica, session=session) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Terminating a session",
                                               "{0}/{1}/sessions/session/{2}".format(uri, replica, session),
                                               requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def delete_all(isamAppliance, replica, username, check_mode=False, force=False):
    """
    Terminating all sessions for a single user
    """

    if force is True or search(isamAppliance=isamAppliance, replica=replica, username=username) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Terminating all sessions for a single user",
                                               "{0}/{1}/sessions/user/{2}".format(uri, replica, username),
                                               requires_modules=requires_modules,
                                               requires_version=requires_version)

    return isamAppliance.create_return_object()


def search(isamAppliance, replica, session=None, username=None, check_mode=False, force=False):
    if session != None:
        ret_obj = get_session(isamAppliance, replica)
        data = ret_obj['data']['matched_sessions']
        for obj in data:
            if obj['session'] == session:
                return True
    elif username != None:
        ret_obj = get_session(isamAppliance, replica, user=username, max=1)
        data = ret_obj['data']['matched_sessions']
        if len(data) == 1:
            return True

    return False



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/logging"
requires_modules = None
requires_versions = None


def get(isamAppliance):
    """
    Retrieves logging configuration attributes
    """
    return isamAppliance.invoke_get("Retrieving logging configuration attributes", module_uri,
                                    requires_modules=requires_modules,
                                    requires_version=requires_versions)


def update(isamAppliance, local, remote_address, remote_port, remote_facility, check_mode=False, force=False):
    """
    Updates logging configuration
    """
    change_required, json_data = _check_update(isamAppliance, local, remote_address, remote_port, remote_facility)

    if force is True or change_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating logging configuration attributes", module_uri, json_data,
                                            requires_modules=requires_modules, requires_version=requires_versions)

    else:
        return isamAppliance.create_return_object()


def _check_update(isamAppliance, local, remote_address, remote_port, remote_facility):
    """
    Checks update for idempotency
    """
    change_required = False
    check_obj = get(isamAppliance)

    # checks to see if logging is local or remote, if remote all other data gets set to None
    if check_obj['data']['local'] is True:
        if check_obj['data']['local'] != local:
            change_required = True

        json_data = {
            "local": True,
            "remote_address": None,
            "remote_port": None,
            "remote_facility": None
        }

    else:
        json_data = {
            "local": local,
            "remote_address": remote_address,
            "remote_port": remote_port,
            "remote_facility": remote_facility
        }
        if check_obj['data']['local'] != local:
            change_required = True
            return change_required, json_data
        if check_obj['data']['remote_address'] != remote_address:
            change_required = True
            return change_required, json_data
        if check_obj['data']['remote_port'] != remote_port:
            change_required = True
            return change_required, json_data
        if check_obj['data']['remote_facility'] != remote_facility:
            change_required = True
            return change_required, json_data

        else:
            return change_required, json_data


def compare(isamAppliance1, isamAppliance2):
    """
    Compare configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import os.path

logger = logging.getLogger(__name__)


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the general configuration
    """
    return isamAppliance.invoke_get("Retrieving the general configuration", "/isam/packet_tracing")


def execute(isamAppliance, operation, enabled, filter=None, interface=None, max_size=None, check_mode=False,
            force=False):
    """
    Execute an operation (start, stop or restart) on packet tracing
    """
    warnings = []

    if force is True or _check(isamAppliance, enabled) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_put(
                "Executing an operation on packet trace", "/isam/packet_tracing/",
                {
                    "enable": enabled,
                    "filter": filter,
                    "interface": interface,
                    "max_size": max_size
                }, warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def delete(isamAppliance, check_mode=False, force=False):
    """
    Execute an operation (start, stop or restart) on packet tracing
    """
    warnings = ["No idempotency check coded yet."]

    if force is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_delete(
                "Executing  delete operation on packet trace", "/isam/packet_tracing/", warnings=warnings)

    return isamAppliance.create_return_object(warnings=warnings)


def export_file(isamAppliance, filepath, filename, check_mode=False, force=False):
    """
    Exporting the packet tracing PCAP file
    """
    if force is True or os.path.exists(filepath) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_get_file(
                "Exporting the packet tracing PCAP file",
                "/isam/packet_tracing/pcap/{0}?export".format(filename), filepath
            )

    return isamAppliance.create_return_object()


def _check(isamAppliance, enabled):
    ret_obj = get(isamAppliance)

    return ret_obj['data']['enabled'] == enabled



import logging
import ibmsecurity.utilities.tools
import os.path
from ibmsecurity.isam.web.reverse_proxy.management_root import directory
from ibmsecurity.isam.web.reverse_proxy.management_root import file

logger = logging.getLogger(__name__)


def export_zip(isamAppliance, instance_id, filename, check_mode=False, force=False):
    """
    Exporting the contents of the administration pages root as a .zip file

    :param isamAppliance:
    :param instance_id:
    :param filename:
    :param check_mode:
    :param force:
    :return:
    """
    if force is True or os.path.exists(filename) is False:
        if check_mode is False:
            return isamAppliance.invoke_get_file(
                "Exporting the contents of the administration pages root as a .zip file",
                "/wga/reverseproxy/{0}/management_root?index=&name=&enc_name=&type=&browser=".format(instance_id),
                filename)

    return isamAppliance.create_return_object()


def import_zip(isamAppliance, instance_id, filename, check_mode=False, force=False):
    """
    Importing the contents of a .zip file to the administration pages root
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_post_files(
            "Importing the contents of a .zip file to the administration pages root",
            "/wga/reverseproxy/{0}/management_root".format(instance_id),
            [
                {
                    'file_formfield': 'file',
                    'filename': filename,
                    'mimetype': 'application/octet-stream'
                }
            ],
            {
                'type': 'file',
                'force': force
            })


def check(isamAppliance, instance_id, id, name, type, check_mode=False, force=False):
    ret_obj = None

    if (type.lower() == 'directory'):
        ret_obj = directory._check(isamAppliance, instance_id, id, name)
    elif (type.lower() == 'file'):
        ret_obj = file._check(isamAppliance, instance_id, id, name)
    else:
        type = 'unknown'

    data = {
        'id': ret_obj,
        'top': id,
        'name': name,
        'type': type
    }

    return isamAppliance.create_return_object(data=data)



import ibmsecurity.utilities.tools
import logging

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/services/"
requires_modulers = None
requires_version = None


def add(isamAppliance, service_name, address, active, port, weight, secure, ssllabel, check_mode=False, force=False):
    """
    Creating a server
    """
    change_required = _check_exist(isamAppliance, service_name, address, port=port)

    if force is True or change_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_post("Creating a server",
                                             "{0}{1}/servers".format(module_uri, service_name, address),
                                             {
                                                 "active": active,
                                                 "address": address,
                                                 "port": port,
                                                 "weight": weight,
                                                 "secure": secure,
                                                 "ssllabel": ssllabel

                                             },
                                             requires_version=requires_version, requires_modules=requires_modulers)
    else:
        return isamAppliance.create_return_object()


def delete(isamAppliance, service_name, address, check_mode=False, force=False):
    """
    deletes a server from specified service name
    """
    if force is True or _check_exist(isamAppliance, service_name, address) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting a server",
                                               "{0}{1}/servers/{2}".format(module_uri, service_name, address),
                                               requires_version=requires_version, requires_modules=requires_modulers)

    else:
        return isamAppliance.create_return_object()


def get(isamAppliance, service_name, address, check_mode=False, force=False):
    """
    Retrieves server from specified service name
    """
    return (
        isamAppliance.invoke_get("Retrieving a server", "{0}{1}/servers/{2}".format(module_uri, service_name, address),
                                 requires_version=requires_version, requires_modules=requires_modulers))


def get_all(isamAppliance, service_name, check_mode=False, force=False):
    """
    Retrieves a list of servers under a specified service
    """
    return isamAppliance.invoke_get("Retrieving servers for a service",
                                    "{0}{1}/servers".format(module_uri, service_name),
                                    requires_version=requires_version, requires_modules=requires_modulers)


def update(isamAppliance, service_name, address, active, new_address, new_port, weight, secure=False, ssllabel=None,
           check_mode=False,
           force=False):
    """
    Updating server
    """
    change_required = _check_update(isamAppliance, service_name, address, active, new_address, new_port, weight, secure,
                                    ssllabel)

    if force is True or change_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:

            return isamAppliance.invoke_put("Updating a server",
                                            "{0}{1}/servers/{2}".format(module_uri, service_name, address),
                                            {
                                                "address": new_address,
                                                "active": active,
                                                "port": new_port,
                                                "weight": weight,
                                                "secure": secure,
                                                "ssllabel": ssllabel

                                            },
                                            requires_modules=requires_modulers,
                                            requires_version=requires_version)
    else:
        return isamAppliance.create_return_object()


def _check_update(isamAppliance, service_name, address, active, new_address, new_port, weight, secure=False,
                  ssllabel=None):
    """
    idempontency test
    """
    org_obj = get(isamAppliance, service_name, address)

    if org_obj['data']['address'] != new_address:
        return True
    elif org_obj['data']['active'] != active:
        return True
    elif org_obj['data']['port'] != new_port:
        return True
    elif org_obj['data']['weight'] != weight:
        return True
    elif org_obj['data']['secure'] != secure:
        return True
    elif org_obj['data']['ssllabel'] != ssllabel:
        return True
    else:
        return False


def _check_exist(isamAppliance, service_name, address):
    """
    idempotency test for delete function
    """
    check_obj = {}
    # Check weather the address with corresponding server exists
    try:
        check_obj = get(isamAppliance, service_name, address)
    except:
        return False

    return True


def compare(isamAppliance1, isamAppliance2):
    """
    Compare cluster configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging

logger = logging.getLogger(__name__)

# URI for this module
uri = "/events/system"
requires_modules = None
requires_version = None


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Exporting the System Event Log

    """
    return isamAppliance.invoke_get("Exporting the System Event Log",
                                    "{0}".format(uri),
                                    requires_modules=requires_modules, requires_version=requires_version)


def get(isamAppliance, field, filter, check_mode=False, force=False):
    """
    Exporting the filtered System Event Log

    """
    return isamAppliance.invoke_get("Exporting the filtered System Event Log",
                                    "{0}/{1}/{2}".format(uri, field, filter),
                                    requires_modules=requires_modules, requires_version=requires_version)



import json
import logging

logger = logging.getLogger(__name__)


def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the general configuration
    """
    return isamAppliance.invoke_get("Retrieving the general configuration", "/net/general")


def set(isamAppliance, hostname, check_mode=False, force=False):
    """
    Updating the general configuration
    """
    if force is True or _check(isamAppliance, hostname) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating the general configuration", "/net/general",
                                            {
                                                'hostName': hostname
                                            })

    return isamAppliance.create_return_object()


def _check(isamAppliance, hostname):
    ret_obj = get(isamAppliance)
    return ret_obj['data']['hostName'] == hostname



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)

module_uri = "/isam/felb/configuration/services/"
requires_modules = None
requires_versions = None


def add(isamAppliance, service_name, name, value, check_mode=False, force=False):
    """
    Creates a service level attribute
    """
    if force is True or _check_add(isamAppliance, service_name, name, value) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Creating a service attribute",
                                            "{0}{1}?uiCalled=True".format(isamAppliance, service_name),
                                            {
                                                "name": name,
                                                "value": value
                                            }, requires_version=requires_versions,
                                            requires_modules=requires_modules)
    else:
        return isamAppliance.create_return_object()


def delete(isamAppliance, service_name, attribute_name, check_mode=False, force=False):
    """
    deletes a service level attribute
    """

    if force is True or _check_delete(isamAppliance, service_name, attribute_name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete("Deleting a service attribute",
                                               "{0}{1}/attributes/{2}".format(isamAppliance, service_name,
                                                                              attribute_name),
                                               requires_version=requires_versions,
                                               requires_modules=requires_modules)
    else:
        return isamAppliance.create_return_object()


def get(isamAppliance, service_name, attribute_name):
    """
    Retrieving a service attribute
    """

    return isamAppliance.invoke_get("Retrieving a service attribute",
                                    "{0}{1}/attributes/{2}".format(module_uri, service_name, attribute_name,
                                                                   requires_version=requires_versions,
                                                                   requires_modules=requires_modules))


def get_all(isamAppliance, service_name):
    """
    Retrieving service attribute names
    """
    return isamAppliance.invoke_get("Retrieving service attribute names",
                                    "{0}{1}/attributes?includeAllValues=true".format(isamAppliance, service_name),
                                    requires_version=requires_versions,
                                    requires_modules=requires_modules)  # TODO REST api matches call used in appliance, returns "failed to parse"


def update(isamAppliance, service_name, attribute_name, attribute_value, check_mode=False, force=False):
    """
    Updating a service attribute
    """
    if force is True or _check_add(isamAppliance, service_name, attribute_name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating a service attribute",
                                            "{0}{1}/attributes/{2}".format(module_uri, service_name, attribute_name),
                                            {
                                                "value": attribute_value
                                            },
                                            requires_modules=requires_modules, requires_version=requires_versions)
    else:
        return isamAppliance.create_return_object()


def set(isamAppliance, service_name, attribute_name, attribute_value, check_mode=False, force=False):
    """
    Determines if add or update is called
    """
    try:
        check_obj = get(isamAppliance, service_name, attribute_name)
    except:
        logger.warning("The attribute does not exist")
        return add(isamAppliance, service_name, attribute_name, attribute_value, check_mode, force)

    return update(isamAppliance, service_name, attribute_name, attribute_value, check_mode, force)


def compare(isamAppliance1, isamAppliance2):
    """
    Compare configuration between two appliances
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])


def _check_add(isamAppliance, service_name, name, value):
    """
    idempotency test for add function
    """
    check_obj = {}
    # check to see if attribute under service name exist, return True if it doesnt exist
    try:
        check_obj = get(isamAppliance, service_name, name)
    except:
        return True

    if check_obj['data']['value'] != value:
        return True
    else:
        return False


def _check_delete(isamAppliance, service_name, attribute_name):
    """
    Checks to see if attribute getting deleted exists
    """
    try:
        check_obj = get(isamAppliance, service_name, attribute_name)
    except:
        return False

    return True



import logging
import os.path
from ibmsecurity.isam.aac.runtime_template import directory
from ibmsecurity.isam.aac.runtime_template import file

logger = logging.getLogger(__name__)

uri = "/mga/template_files"

requires_modules = ["mga", "federation"]
requires_version = None


def export_file(isamAppliance, filename, check_mode=False, force=False):
    """
    Export all Runtime Template Files
    """
    import os.path

    if force is True or os.path.exists(filename) is False:
        if check_mode is False:  # No point downloading a file if in check_mode
            return isamAppliance.invoke_get_file(
                "Export all Runtime Template Files",
                "{0}/?export=true".format(uri),
                filename, no_headers=True, requires_modules=requires_modules,
                requires_version=requires_version)

    return isamAppliance.create_return_object()


def import_file(isamAppliance, filename, check_mode=False, force=False):
    """
    Replace all Runtime Template Files
    """
    if check_mode is True:
        return isamAppliance.create_return_object(changed=True)
    else:
        return isamAppliance.invoke_post_files(
            "Replace all Runtime Template Files",
            uri,
            [
                {
                    'file_formfield': 'file',
                    'filename': filename,
                    'mimetype': 'application/octet-stream'
                }
            ],
            {
                "force": force
            }, json_response=False, requires_modules=requires_modules,
            requires_version=requires_version)


def check(isamAppliance, id, type, check_mode=False, force=False):
    ret_obj = None

    if (type.lower() == 'directory'):
        ret_obj = directory._check(isamAppliance, id)
    elif (type.lower() == 'file'):
        ret_obj = file._check(isamAppliance, id)
    else:
        type = 'unknown'

    name = os.path.basename(id)
    path = os.path.dirname(id)

    data = {
        'id': ret_obj,
        'path': path,
        'name': name,
        'type': type
    }

    return isamAppliance.create_return_object(data=data)


def delete(isamAppliance, id, type, check_mode=False, force=False):
    """
    Deleting a file or directory in the runtime template files directory

    :param isamAppliance:
    :param id:
    :param_type:
    :param check_mode:
    :param force:
    :return:
    """
    if (type.lower() == 'directory'):
        return directory.delete(isamAppliance, id)
    elif (type.lower() == 'file'):
        return file.delete(isamAppliance, id)



import logging

logger = logging.getLogger(__name__)

# URI for this module
uri = "/iam/access/v8/override-configs"
requires_modules = ["mga", "federation"]
requires_version = None


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Retrieve a list of configuration properties
    """
    return isamAppliance.invoke_get("Retrieve a list of configuration properties",
                                    "/iam/access/v8/override-configs", requires_modules=requires_modules,
                                    requires_version=requires_version)


def get(isamAppliance, key, check_mode=False, force=False):
    """
    Retrieve a specific configuration property
    """
    ret_obj = search(isamAppliance, key=key)
    id = ret_obj['data']

    if id == {}:
        return isamAppliance.create_return_object()
    else:
        return isamAppliance.invoke_get("Retrieve a specific configuration property",
                                        "/iam/access/v8/override-configs/{0}".format(id),
                                        requires_modules=requires_modules, requires_version=requires_version)


def update(isamAppliance, key, value, sensitive, check_mode=False, force=False):
    """
    Update a configuration property
    """
    id, matches = None, None
    if force is False:
        id, matches = _check(isamAppliance, key, value, sensitive)

    if force is True or (matches is False and id is not None):
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put(
                "Update a configuration property",
                "/iam/access/v8/override-configs/{0}".format(id),
                {
                    'value': value,
                    'sensitive': sensitive
                }, requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def update_all(isamAppliance, values, check_mode=False, force=False):
    """
    Update a list of configuration properties

    Unlike the REST API use key values instead of id
    """
    matches, vals = None, values
    if force is False:
        matches, vals = _check_values(isamAppliance, values)

    if force is True or matches is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put(
                "Update a list of configuration properties",
                "/iam/access/v8/override-configs",
                vals, requires_modules=requires_modules, requires_version=requires_version)

    return isamAppliance.create_return_object()


def _check(isamAppliance, key, value, sensitive):
    """
    Check and see if given key value matches existing values to be updated and also
    extract the id to use instead of the key
    """
    id, matches = None, None
    ret_obj = get_all(isamAppliance)

    for obj in ret_obj['data']:
        if obj['key'] == key:
            id = obj['id']
            if obj['value'] == value and obj['sensitive'] == sensitive:
                matches = True
            else:
                matches = False
            break

    return id, matches


def _check_values(isamAppliance, values):
    """
    Filter out values to be updated, and evaluate if an update needs to happen

    Note: Key values not found will be skipped!
    """
    matches, vals = True, []
    ret_obj = get_all(isamAppliance)

    for new_obj in values:
        for obj in ret_obj['data']:
            if obj['key'] == new_obj['key']:
                if obj['value'] != new_obj['value'] or obj['sensitive'] != new_obj['sensitive']:
                    matches = False
                    vals.append({'id': obj['id'], 'value': new_obj['value'], 'sensitive': new_obj['sensitive']})
                break

    logger.debug(vals)

    return matches, vals


def search(isamAppliance, key):
    """
    Get the ID for a given key value
    """
    ret_obj = get_all(isamAppliance)

    ret_obj_new = isamAppliance.create_return_object()

    for obj in ret_obj['data']:
        if obj['key'] == key:
            ret_obj_new['data'] = obj['id']
            break

    return ret_obj_new


def compare(isamAppliance1, isamAppliance2):
    """
    Compare AAC Advanced configuration between two appliances
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    for obj in ret_obj1['data']:
        del obj['id']
    for obj in ret_obj2['data']:
        del obj['id']

    import ibmsecurity.utilities.tools
    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['id'])
